use std::f32::consts::TAU;

use crate::camera::Watched;
use crate::movement::{Heading, Mover, Speed, Velocity, YSortable};
use crate::prelude::*;
use bevy::sprite::MaterialMesh2dBundle;

const PLAYER_SPEED: f32 = 200.;

#[derive(Component, Debug)]
pub struct Player;

#[derive(Component, Debug)]
pub struct PlayerSprite;

#[derive(Component, Debug)]
pub struct Crosshair;

pub fn startup(
    mut commands: Commands,
    assets: Res<AssetServer>,
    mut meshes: ResMut<Assets<Mesh>>,
    mut materials: ResMut<Assets<ColorMaterial>>,
) {
    let mesh = meshes.add(Capsule2d::new(3.0, 25.0));
    let material = materials.add(Color::hsla(360. * 1 as f32 / 3 as f32, 0.95, 0.7, 0.5));
    let transform = Transform::from_xyz(0.0, 0.0, 500.);
    let layout = assets.add(TextureAtlasLayout::from_grid(
        UVec2::new(32, 64),
        3,
        1,
        Some(UVec2 { x: 0, y: 0 }),
        Some(UVec2 { x: 0, y: 0 }),
    ));
    commands
        .spawn((
            Player,
            Name::new("Player"),
            VisibilityBundle {
                visibility: Visibility::Inherited,
                ..default()
            },
            YSortable,
            Watched,
            TransformBundle::default(),
            SpatialListener::new(20.),
            Mover {
                velocity: Velocity(Vec2::ZERO),
                heading: Heading(Vec2::ZERO),
                speed: Speed(PLAYER_SPEED),
            },
        ))
        .with_children(|player| {
            player.spawn((
                PlayerSprite,
                Name::new("PlayerSprite"),
                SpriteBundle {
                    texture: assets.load("img/Player.png"),
                    ..default()
                },
                TextureAtlas {
                    layout,
                    ..default()
                },
            ));
            let mut text = Text2dBundle {
                text: Text::from_section(
                    "You",
                    TextStyle {
                        font_size: 16. * 4.,
                        color: Color::WHITE,
                        ..default()
                    },
                ),
                ..default()
            };
            text.transform.scale.x = 0.25;
            text.transform.scale.y = 0.25;
            text.transform.translation.z = 500.0;
            text.transform.translation.y = 32.;
            player.spawn((Name::new("PlayerName"), text));
            player
                .spawn((
                    Crosshair,
                    Name::new("Crosshair"),
                    TransformBundle {
                        local: transform,
                        ..default()
                    },
                ))
                .with_children(|crosshair| {
                    crosshair.spawn((
                        Name::new("CrosshairMesh"),
                        MaterialMesh2dBundle {
                            mesh: mesh.into(),
                            transform: Transform::from_xyz(0., 20., 0.),
                            material,
                            ..default()
                        },
                    ));
                });
        });
}

pub fn control(
    input: Res<Input>,
    mut crosshair: Query<&mut Transform, With<Crosshair>>,
    mut player: Query<&mut Heading, With<Player>>,
    mut sprite: Query<(&mut Sprite, &mut TextureAtlas), With<PlayerSprite>>,
    // time: Res<Time>,
) {
    if let Ok(mut heading) = player.get_single_mut() {
        heading.0 = input.movement;
        if let Ok((mut sprite, mut texture)) = sprite.get_single_mut() {
            if heading.y < 0. {
                texture.index = 0;
            } else if heading.y > 0. {
                texture.index = 1;
            } else if heading.x != 0. {
                texture.index = 2;
                sprite.flip_x = heading.x > 0.;
            }
        }
    }
    if let Ok(mut transform) = crosshair.get_single_mut() {
        transform.rotation = Quat::from_rotation_z(input.angle - (TAU / 4.));
    }
}

pub fn meow_on_r(
    mut commands: Commands,
    assets: Res<AssetServer>,
    player: Query<Entity, With<Player>>,
    keys: Res<ButtonInput<KeyCode>>,
) {
    if keys.just_pressed(KeyCode::KeyR) {
        if let Ok(player) = player.get_single() {
            let sound = commands
                .spawn((
                    SpatialBundle::default(),
                    AudioBundle {
                        source: assets.load::<AudioSource>("sfx/meow.wav"),
                        settings: PlaybackSettings::DESPAWN.with_spatial(true),
                        ..default()
                    },
                ))
                .id();
            commands.entity(player).push_children(&[sound]);
        }
    }
}

pub fn player_debug_info(mut player: Query<(&Heading, &Transform), With<Player>>) {
    if let Ok((_heading, _transform)) = player.get_single_mut() {
        // info!("{heading:#?} {transform:#?}");
    }
}

pub fn exit(mut commands: Commands, q: Query<Entity, With<Player>>) {
    for id in q.iter() {
        commands.entity(id).despawn_recursive();
    }
}
